# -*- coding: UTF-8 -*-
"""
  Author:  Jacek Kotlarski --<szumak@virthost.pl>
  Created: 05.09.2023

  Purpose: various classes of interaction with the system.
"""

import os
from inspect import currentframe
from pathlib import Path
from typing import Optional, List

from jsktoolbox.attribtool import NoDynamicAttributes
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData


class Env(NoDynamicAttributes):
    """Environment class."""

    @classmethod
    @property
    def home(cls) -> str:
        """Return home dir name."""
        return os.getenv("HOME")

    @classmethod
    @property
    def username(cls) -> str:
        """Return login name."""
        return os.getenv("USER")


class PathChecker(BData, NoDynamicAttributes):
    """PathChecker class for filesystem path."""

    def __init__(self, pathname: str, check_deep: bool = True) -> None:
        """Constructor."""
        if pathname is None:
            raise Raise.error(
                "pathname as string expected, not None.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        if not isinstance(pathname, str):
            raise Raise.error(
                f"pathname as string expected, '{type(pathname)}' received.",
                TypeError,
                self.__class__.__name__,
                currentframe(),
            )
        if isinstance(pathname, str) and len(pathname) == 0:
            raise Raise.error(
                "pathname cannot be an empty string.",
                ValueError,
                self.__class__.__name__,
                currentframe(),
            )
        self._data["pathname"] = pathname
        self._data["split"] = check_deep
        self._data["list"]: List = []
        # analysis
        self.__run__()

    def __run__(self) -> None:
        """Path analysis procedure."""
        query = Path(self._data["pathname"])
        # check exists
        self._data["exists"] = query.exists()
        if self._data["exists"]:
            # check isfile
            self._data["isfile"] = query.is_file()
            # check isdir
            self._data["isdir"] = query.is_dir()
            # check issymlink
            self._data["issymlink"] = query.is_symlink()
            # resolve symlink
            self._data["posixpath"] = str(query.resolve())

        if self._data["split"]:
            # split and analyse
            tmp = ""
            for item in self.path.split(os.sep):
                if item == "":
                    continue
                tmp += f"{os.sep}{item}"
                self._data["list"].append(PathChecker(tmp, False))

    def __str__(self) -> str:
        """Return class data as string."""
        return (
            "PathChecker("
            f"'pathname': '{self.path}', "
            f"'exists': '{self.exists}', "
            f"'is_dir': '{self.is_dir if self.exists else ''}', "
            f"'is_file': '{self.is_file if self.exists else ''}', "
            f"'is_symlink': '{self.is_symlink if self.exists else ''}', "
            f"'posixpath': '{self.posixpath if self.exists else ''}'"
            ")"
        )

    def __repr__(self) -> str:
        """Return string representation."""
        return f"PathChecker('{self.path}')"

    @property
    def dirname(self) -> Optional[str]:
        """Return dirname from path."""
        if self.exists:
            last = None
            for item in self._data["list"]:
                if item.is_dir:
                    last = item.path
            return last
        return None

    @property
    def filename(self) -> Optional[str]:
        """Return filename from path."""
        if self.exists and self.is_file:
            tmp = self.path.split(os.sep)
            if len(tmp) > 0:
                if tmp[-1] != "":
                    return tmp[-1]
        return None

    @property
    def exists(self) -> bool:
        """Return path exists flag."""
        if "exists" in self._data:
            return self._data["exists"]
        else:
            raise Raise.error(
                "Unexpected exception",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )

    @property
    def is_dir(self) -> bool:
        """Return path isdir flag."""
        if "isdir" in self._data:
            return self._data["isdir"]
        else:
            raise Raise.error(
                "Unexpected exception",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )

    @property
    def is_file(self) -> bool:
        """Return path isfile flag."""
        if "isfile" in self._data:
            return self._data["isfile"]
        else:
            raise Raise.error(
                "Unexpected exception",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )

    @property
    def is_symlink(self) -> bool:
        """Return path issymlink flag."""
        if "issymlink" in self._data:
            return self._data["issymlink"]
        else:
            raise Raise.error(
                "Unexpected exception",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )

    @property
    def path(self) -> str:
        """Return path string."""
        return self._data["pathname"]

    @property
    def posixpath(self) -> Optional[str]:
        """Return path string."""
        if self.exists:
            return self._data["posixpath"]
        return None

    def create(self) -> bool:
        """Create path procedure."""
        test_path = self.path
        file = True
        if self.path[-1] == os.sep:
            file = False
            test_path = self.path[:-1]
        for item in self._data["list"]:
            if item.exists:
                continue
            if item.path == test_path:
                # last element
                if file:
                    # touch file
                    with open(item.path, "w") as fp:
                        pass
                else:
                    os.mkdir(item.path)
            else:
                os.mkdir(item.path)
        # check
        self._data["list"] = []
        self.__run__()

        return self.exists


# #[EOF]#######################################################################



# -*- coding: UTF-8 -*-
"""
  Author:  Jacek Kotlarski --<szumak@virthost.pl>
  Created: 04.09.2023

  Purpose: base class for log subsystem.
"""

import syslog

from inspect import currentframe
from typing import Optional, Tuple, List, Dict, Any

from jsktoolbox.attribtool import NoDynamicAttributes
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData


class Keys(NoDynamicAttributes):
    """Keys definition class.

    For internal purpose only.
    """

    @classmethod
    @property
    def BUFFERED(cls) -> str:
        """Return BUFFERED Key."""
        return "__buffered__"

    @classmethod
    @property
    def CONF(cls) -> str:
        """Return CONF Key."""
        return "__conf__"

    @classmethod
    @property
    def DIR(cls) -> str:
        """Return DIR Key."""
        return "__dir__"

    @classmethod
    @property
    def FILE(cls) -> str:
        """Return FILE Key."""
        return "__file__"

    @classmethod
    @property
    def FORMATTER(cls) -> str:
        """Return FORMATTER Key."""
        return "__formatter__"

    @classmethod
    @property
    def FACILITY(cls) -> str:
        """Return FACILITY Key."""
        return "__facility__"

    @classmethod
    @property
    def LEVEL(cls) -> str:
        """Return LEVEL Key."""
        return "__level__"

    @classmethod
    @property
    def NAME(cls) -> str:
        """Return NAME Key."""
        return "__name__"

    @classmethod
    @property
    def NO_CONF(cls) -> str:
        """Return NO_CONF Key."""
        return "__noconf__"

    @classmethod
    @property
    def QUEUE(cls) -> str:
        """Return QUEUE Key."""
        return "__queue__"

    @classmethod
    @property
    def SYSLOG(cls) -> str:
        """Return SYSLOG Key."""
        return "__syslog__"


class SysLogKeys(NoDynamicAttributes):
    """SysLog keys definition container class."""

    class __Levels(NoDynamicAttributes):
        @classmethod
        @property
        def NOTICE(cls) -> int:
            return syslog.LOG_NOTICE

        @classmethod
        @property
        def EMERGENCY(cls) -> int:
            return syslog.LOG_EMERG

        @classmethod
        @property
        def ALERT(cls) -> int:
            return syslog.LOG_ALERT

        @classmethod
        @property
        def CRITICAL(cls) -> int:
            return syslog.LOG_CRIT

        @classmethod
        @property
        def INFO(cls) -> int:
            return syslog.LOG_INFO

        @classmethod
        @property
        def DEBUG(cls) -> int:
            return syslog.LOG_DEBUG

        @classmethod
        @property
        def WARNING(cls) -> int:
            return syslog.LOG_WARNING

        @classmethod
        @property
        def ERROR(cls) -> int:
            return syslog.LOG_ERR

    class __Facilities(NoDynamicAttributes):
        @classmethod
        @property
        def DAEMON(cls) -> int:
            return syslog.LOG_DAEMON

        @classmethod
        @property
        def USER(cls) -> int:
            return syslog.LOG_USER

        @classmethod
        @property
        def LOCAL0(cls) -> int:
            return syslog.LOG_LOCAL0

        @classmethod
        @property
        def LOCAL1(cls) -> int:
            return syslog.LOG_LOCAL1

        @classmethod
        @property
        def LOCAL2(cls) -> int:
            return syslog.LOG_LOCAL2

        @classmethod
        @property
        def LOCAL3(cls) -> int:
            return syslog.LOG_LOCAL3

        @classmethod
        @property
        def LOCAL4(cls) -> int:
            return syslog.LOG_LOCAL4

        @classmethod
        @property
        def LOCAL5(cls) -> int:
            return syslog.LOG_LOCAL5

        @classmethod
        @property
        def LOCAL6(cls) -> int:
            return syslog.LOG_LOCAL6

        @classmethod
        @property
        def LOCAL7(cls) -> int:
            return syslog.LOG_LOCAL7

        @classmethod
        @property
        def MAIL(cls) -> int:
            return syslog.LOG_MAIL

        @classmethod
        @property
        def SYSLOG(cls) -> int:
            return syslog.LOG_SYSLOG

    @classmethod
    @property
    def level(cls):
        """Returns Levels keys property."""
        return cls.__Levels

    @classmethod
    @property
    def facility(cls):
        """Returns Facility keys property."""
        return cls.__Facilities

    @classmethod
    @property
    def level_keys(cls) -> Dict:
        """Returns level keys property."""
        return {
            "NOTICE": SysLogKeys.level.NOTICE,
            "INFO": SysLogKeys.level.INFO,
            "DEBUG": SysLogKeys.level.DEBUG,
            "WARNING": SysLogKeys.level.WARNING,
            "ERROR": SysLogKeys.level.ERROR,
            "EMERGENCY": SysLogKeys.level.EMERGENCY,
            "ALERT": SysLogKeys.level.ALERT,
            "CRITICAL": SysLogKeys.level.CRITICAL,
        }

    @classmethod
    @property
    def facility_keys(cls) -> Dict:
        """Returns Facility keys property."""
        return {
            "DAEMON": SysLogKeys.facility.DAEMON,
            "USER": SysLogKeys.facility.USER,
            "LOCAL0": SysLogKeys.facility.LOCAL0,
            "LOCAL1": SysLogKeys.facility.LOCAL1,
            "LOCAL2": SysLogKeys.facility.LOCAL2,
            "LOCAL3": SysLogKeys.facility.LOCAL3,
            "LOCAL4": SysLogKeys.facility.LOCAL4,
            "LOCAL5": SysLogKeys.facility.LOCAL5,
            "LOCAL6": SysLogKeys.facility.LOCAL6,
            "LOCAL7": SysLogKeys.facility.LOCAL7,
            "MAIL": SysLogKeys.facility.MAIL,
            "SYSLOG": SysLogKeys.facility.SYSLOG,
        }


class LogsLevelKeys(NoDynamicAttributes):
    """LogsLevelKeys container class."""

    @classmethod
    @property
    def keys(cls) -> Tuple[str]:
        """Return tuple of avaiable keys."""
        return tuple(
            [
                LogsLevelKeys.ALERT,
                LogsLevelKeys.CRITICAL,
                LogsLevelKeys.DEBUG,
                LogsLevelKeys.EMERGENCY,
                LogsLevelKeys.ERROR,
                LogsLevelKeys.INFO,
                LogsLevelKeys.NOTICE,
                LogsLevelKeys.WARNING,
            ]
        )

    @classmethod
    @property
    def ALERT(cls) -> str:
        """Return ALERT Key."""
        return "ALERT"

    @classmethod
    @property
    def CRITICAL(cls) -> str:
        """Return CRITICAL Key."""
        return "CRITICAL"

    @classmethod
    @property
    def DEBUG(cls) -> str:
        """Return DEBUG Key."""
        return "DEBUG"

    @classmethod
    @property
    def EMERGENCY(cls) -> str:
        """Return EMERGENCY Key."""
        return "EMERGENCY"

    @classmethod
    @property
    def ERROR(cls) -> str:
        """Return ERROR Key."""
        return "ERROR"

    @classmethod
    @property
    def INFO(cls) -> str:
        """Return INFO Key."""
        return "INFO"

    @classmethod
    @property
    def NOTICE(cls) -> str:
        """Return NOTICE Key."""
        return "NOTICE"

    @classmethod
    @property
    def WARNING(cls) -> str:
        """Return WARNING Key."""
        return "WARNING"


class LoggerQueue(NoDynamicAttributes):
    """LoggerQueue simple class."""

    __queue: List[str] = None

    def __init__(self):
        """Constructor."""
        self.__queue = []

    def get(self) -> Optional[Tuple[str, str]]:
        """Get item from queue.

        Returs queue tuple[log_level:str, message:str] or None if empty.
        """
        try:
            return tuple(self.__queue.pop(0))
        except IndexError:
            return None
        except Exception as ex:
            raise Raise.error(
                f"Unexpected exception was thrown: {ex}",
                self.__class__.__name__,
                currentframe(),
            )

    def put(self, message: str, log_level: str = LogsLevelKeys.INFO) -> None:
        """Put item to queue."""
        if log_level not in LogsLevelKeys.keys:
            raise Raise.error(
                f"logs_level key not found, '{log_level}' received.",
                KeyError,
                self.__class__.__name__,
                currentframe(),
            )
        self.__queue.append(
            [
                log_level,
                message,
            ]
        )


class BLoggerQueue(BData, NoDynamicAttributes):
    """Logger Queue base metaclass."""

    @property
    def logs_queue(self) -> Optional[LoggerQueue]:
        """Get LoggerQueue object."""
        if Keys.QUEUE not in self._data:
            return None
        return self._data[Keys.QUEUE]

    @logs_queue.setter
    def logs_queue(self, obj: LoggerQueue) -> None:
        """Set LoggerQueue object."""
        if not isinstance(obj, LoggerQueue):
            raise Raise.error(
                f"LoggerQueue type object expected, '{type(obj)}' received.",
                self.__class__.__name__,
                currentframe(),
            )
        self._data[Keys.QUEUE] = obj


class BLoggerEngine(BData, NoDynamicAttributes):
    """Base class for LoggerEngine classes."""

    @property
    def name(self) -> Optional[str]:
        """Return app name string."""
        if Keys.NAME not in self._data:
            self._data[Keys.NAME] = None
        return self._data[Keys.NAME]

    @name.setter
    def name(self, value: str) -> None:
        """Set app name string."""
        self._data[Keys.NAME] = value


class BLogFormatter(NoDynamicAttributes):
    """Log formatter base class."""

    __template: Optional[str] = None
    __forms: Optional[List] = None

    def __init__(self) -> None:
        """Constructor."""
        self.__forms = []

    def format(self, message: str, name: str = None) -> str:
        """Method for format message string.

        Arguments:
        message [str]: log string to send
        name [str]: optional name of apps,
        """
        out = ""
        for item in self._forms_:
            if callable(item):
                out += f"{item()} "
            elif isinstance(item, str):
                if name is None:
                    if item.find("name") == -1:
                        out += item.format(message=f"{message}")
                else:
                    if item.find("name") > 0:
                        out += item.format(
                            name=f"{name}",
                            message=f"{message}",
                        )
        return out

    @property
    def _forms_(self) -> List:
        """Get forms list."""
        return self.__forms

    @_forms_.setter
    def _forms_(self, item: Any) -> None:
        """Set forms list."""
        # assigning function to a variable
        # def a(): print('test')
        # var=a
        # var()
        ####
        # >>> x._forms_[2].__class__
        # <class 'builtin_function_or_method'>
        # >>> x._forms_[1].__class__
        # <class 'float'>
        # >>> x._forms_[0].__class__
        # <class 'str'>

        self.__forms.append(item)


# #[EOF]#######################################################################



# -*- coding: UTF-8 -*-
"""
  Author:  Jacek 'Szumak' Kotlarski --<szumak@virthost.pl>
  Created: 29.10.2023

  Purpose: Main class for creating and processes config files.
"""

import re
from inspect import currentframe
from typing import List, Dict, Optional, Any
from jsktoolbox.attribtool import NoDynamicAttributes
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData
from jsktoolbox.configtool.libs.file import FileProcessor
from jsktoolbox.configtool.libs.data import DataProcessor


class Keys(NoDynamicAttributes):
    """Keys definition class.

    For internal purpose only.
    """

    @classmethod
    @property
    def VARNAME(self) -> str:
        """Return FP key."""
        return "__varname__"

    @classmethod
    @property
    def VALUE(self) -> str:
        """Return FP key."""
        return "__value__"

    @classmethod
    @property
    def DESC(self) -> str:
        """Return FP key."""
        return "__desc__"

    @classmethod
    @property
    def FP(self) -> str:
        """Return FP key."""
        return "__file_processor__"

    @classmethod
    @property
    def DP(self) -> str:
        """Return DP key."""
        return "__data_processor__"

    @classmethod
    @property
    def RE_SECTION(self) -> str:
        """Return RE_SECTION key."""
        return "__re_section__"

    @classmethod
    @property
    def RE_VAR(self) -> str:
        """Return RE_VAR key."""
        return "__re_variable__"

    @classmethod
    @property
    def RE_DESC(self) -> str:
        """Return RE_DESC key."""
        return "__re_description__"

    @classmethod
    @property
    def RE_INT(self) -> str:
        """Return RE_INT key."""
        return "__re_integer__"

    @classmethod
    @property
    def RE_FLOAT(self) -> str:
        """Return RE_FLOAT key."""
        return "__re_float__"

    @classmethod
    @property
    def RE_BOOL(self) -> str:
        """Return RE_BOOL key."""
        return "__re_bool__"

    @classmethod
    @property
    def RE_TRUE(self) -> str:
        """Return RE_TRUE key."""
        return "__re_true__"

    @classmethod
    @property
    def RE_FALSE(self) -> str:
        """Return RE_FALSE key."""
        return "__re_false__"

    @classmethod
    @property
    def RE_LIST(self) -> str:
        """Return RE_LIST key."""
        return "__re_list__"


class Config(BData, NoDynamicAttributes):
    """Config main class."""

    def __init__(
        self,
        filename: str,
        main_section_name: str,
        auto_create: bool = False,
    ) -> None:
        """Constructor."""
        self._data[Keys.FP] = FileProcessor()
        self._data[Keys.DP] = DataProcessor()
        self.__fp.file = filename
        self.__dp.main_section = main_section_name
        if auto_create:
            if not self.__fp.file_exists:
                self.__fp.file_create()
        # compile regex
        self._data[Keys.RE_SECTION] = re.compile(r"\s{0,}\[.*\]\s{0,}")
        self._data[Keys.RE_DESC] = re.compile(r"\s{0,}#")
        self._data[Keys.RE_VAR] = re.compile(r"\s{0,}\S{1,}\s{0,}=")
        self._data[Keys.RE_INT] = re.compile(r"^\d{1,}$")
        self._data[Keys.RE_FLOAT] = re.compile(r"^\d{1,}\.\d{1,}$")
        self._data[Keys.RE_BOOL] = re.compile(
            r"^true|false|yes|no$", re.IGNORECASE
        )
        self._data[Keys.RE_TRUE] = re.compile(r"^true|yes$", re.IGNORECASE)
        self._data[Keys.RE_FALSE] = re.compile(r"^false|no$", re.IGNORECASE)
        self._data[Keys.RE_LIST] = re.compile(r"^\[.*\]$")

    @property
    def __fp(self) -> FileProcessor:
        """Return FileProcessor object."""
        return self._data[Keys.FP]

    @property
    def __dp(self) -> DataProcessor:
        """Return DataProcessor object."""
        return self._data[Keys.DP]

    @property
    def file_exists(self) -> bool:
        """Check if file exists."""
        return self.__fp.file_exists

    def __value_parser(self, item: str) -> Any:
        """Return proper type of value."""
        if self._data[Keys.RE_BOOL].match(item):
            return True if self._data[Keys.RE_TRUE].match(item) else False
        elif self._data[Keys.RE_INT].match(item):
            return int(item)
        elif self._data[Keys.RE_FLOAT].match(item):
            return float(item)
        elif self._data[Keys.RE_LIST].match(item):
            out = []
            tmp = [x.strip() for x in item.strip("[]").split(",")]
            for item in tmp:
                out.append(self.__value_parser(item))
            return out
        return str(item.strip("\"'"))

    def __var_parser(self, line: str) -> Dict:
        """Return Dict[varname, value, desc]."""
        out = {
            Keys.VARNAME: None,
            Keys.VALUE: None,
            Keys.DESC: None,
        }
        tmp = line.split("=", 1)
        if len(tmp) != 2:
            raise Raise.error(
                f"Unexpected config line format: '{line}'",
                ValueError,
                self.__class__.__name__,
                currentframe(),
            )
        out[Keys.VARNAME] = tmp[0].strip()
        if len(tmp[1]) > 0:
            tmp = tmp[1].split("#", 1)
            # desc
            if len(tmp) == 2 and len(tmp[1]) > 0:
                out[Keys.DESC] = tmp[1].strip()
            # value
            out[Keys.VALUE] = self.__value_parser(tmp[0].strip())

        return out

    def load(self) -> bool:
        """Load config file to DataProcessor."""
        test = False
        # 1. load file into list
        file: List[str] = self.__fp.readlines()
        section_name: str = self.__dp.main_section
        for line in file:
            # check section
            if self._data[Keys.RE_SECTION].match(line):
                section_name = self.__dp.add_section(line)
            # check description
            elif self._data[Keys.RE_DESC].match(line):
                self.__dp.set(section_name, desc=line.strip("# "))
            # check var
            elif self._data[Keys.RE_VAR].match(line):
                out = self.__var_parser(line)
                self.__dp.set(
                    section=section_name,
                    varname=out[Keys.VARNAME],
                    value=out[Keys.VALUE],
                    desc=out[Keys.DESC],
                )
            else:
                self.__dp.set(section_name, desc=line)
            test = True
        return test

    def save(self) -> bool:
        """Save config file from DataProcessor."""
        test = False
        self.__fp.write(self.__dp.dump)
        test = True
        return test

    def get(
        self, section: str, varname: Optional[str] = None, desc: bool = False
    ) -> Any:
        """Get and return data."""
        return self.__dp.get(section, varname, desc)

    def set(
        self,
        section: str,
        varname: Optional[str] = None,
        value: Optional[Any] = None,
        desc: Optional[str] = None,
    ) -> None:
        """Set data."""
        self.__dp.set(section, varname, value, desc)


# #[EOF]#######################################################################



# -*- coding: UTF-8 -*-
"""
  Author:  Jacek 'Szumak' Kotlarski --<szumak@virthost.pl>
  Created: 29.10.2023

  Purpose: Class for creating and processes config files.
"""

from inspect import currentframe
from typing import List, Optional
from jsktoolbox.attribtool import NoDynamicAttributes
from jsktoolbox.raisetool import Raise
from jsktoolbox.libs.base_data import BData
from jsktoolbox.libs.system import PathChecker


class Keys(NoDynamicAttributes):
    """Keys definition class.

    For internal purpose only.
    """

    @classmethod
    @property
    def FILE(self) -> str:
        """Return FILE key."""
        return "__file__"


class FileProcessor(BData, NoDynamicAttributes):
    """FileProcessor class."""

    def __init__(self) -> None:
        """Constructor."""

    @property
    def file(self) -> Optional[str]:
        """Return config file path."""
        if Keys.FILE not in self._data:
            self._data[Keys.FILE] = None
        if isinstance(self._data[Keys.FILE], PathChecker):
            return self._data[Keys.FILE].path
        return self._data[Keys.FILE]

    @file.setter
    def file(self, path: str) -> None:
        """Set file name."""
        self._data[Keys.FILE] = PathChecker(path)

    @property
    def file_exists(self) -> bool:
        """Check if the file exists and is a file."""
        obj: PathChecker = self._data[Keys.FILE]
        return (
            obj.exists and (obj.is_file or obj.is_symlink) and not obj.is_dir
        )

    def file_create(self) -> bool:
        """Try to create file."""
        if self.file_exists:
            return True
        obj: PathChecker = self._data[Keys.FILE]
        if obj.exists and obj.is_dir:
            raise Raise.error(
                f"Given path: {obj.path} exists and is a directory.",
                OSError,
                self.__class__.__name__,
                currentframe(),
            )
        return obj.create()

    def read(self) -> str:
        """Try to read config file."""
        out = ""
        if self.file_exists:
            with open(self.file, "r") as file:
                out = file.read()
        return out

    def readlines(self) -> List[str]:
        """Try to read config file and create list of strings."""
        out: List[str] = []
        if self.file_exists:
            with open(self.file, "r") as file:
                tmp = file.readlines()
                for line in tmp:
                    if line.find("<End of section") > 0:
                        continue
                    out.append(line.strip())
        return out

    def write(self, data: str) -> None:
        """Try to write data to config file."""
        test = self.file_exists
        if not test:
            test = self.file_create()
        if test:
            with open(self.file, "w") as file:
                file.write(data)


# #[EOF]#######################################################################
